(function () {
  window.SimpleToC = (function () {
    /**
     * Convert header's text to slug
     * @param {string} header: header title
     * @return {string} slug
     */
    function getHeaderSlug(header) {
      return header
        .trim()
        .toLowerCase()
        .replace(/[^a-zA-Z0-9]/g, ' ')
        .replace(/[\s]+/g, '-');
    }

    function getHeaderLinksMap(headers) {
      const links = [];
      [...headers].forEach((header, i) => {
        const text = header.innerText.trim();
        const id = header.getAttribute('id') || `${getHeaderSlug(text)}-${i}`;

        if (text.match(/[\S]+/)) {
          header.setAttribute('id', id);
          links.push({ text, id });
          return true;
        }

        return false;
      });
      return links;
    }

    function getTemplate(linksMap, title) {
      return `<div class="lt-toc-simple">
        <div class="lt-toc-simple__title">${title}</div>
        <ul class="lt-toc-simple__list">${linksMap
      .map(
        (link) => `<li class="lt-toc-simple__item">
            <a class="lt-toc-simple__link" href="#${link.id}">${link.text}</a></li>`,
      )
      .join('')}
        </ul>
      </div>`;
    }

    return {
    /**
     * Initializes the plugin instance.
     *
     * @param options
     */
      init(options) {
        const defaults = {
          contentEl: '.article-body',
          headers: 'h1, h2, h3, h4, h5',
          title: 'In this article',
          tocElementId: '#toc',
        };

        const extOptions = { ...defaults, ...options };
        const contentEl = document.querySelector(extOptions.contentEl);
        if (!contentEl) return;

        const headerEls = contentEl.querySelectorAll(extOptions.headers);
        if (!headerEls.length) return;

        const linksMap = getHeaderLinksMap(headerEls);
        const template = getTemplate(linksMap, extOptions.title);

        const tocEl = contentEl.querySelector(extOptions.tocElementId);
        if (!tocEl) {
          const newTocEl = document.createElement('div');
          newTocEl.setAttribute('id', 'toc');
          newTocEl.innerHTML = template;
          contentEl.insertBefore(newTocEl, contentEl.firstChild);
        } else {
          tocEl.innerHTML = template;
        }
      },
    };
  }());
}());
